
bind PotMeter <Any-Enter> {
	focus -force %W
}

proc addPotMeter { frame name format } {
	global color

	set size 48

	if { $format == "small" } {
		set size 30
	}
	if { $format == "medium" } {
		set size 40
	}
	if { $format == "large" } {
		set size 60
	}

	label $frame.value -textvariable $name -relief sunken -bd 2 -bg black \
		-fg lightblue -width 3 -font 9x15 -padx 2 -pady 2
	potmeter $frame.pot -variable $name -start 128 -bg $color -style bevel \
		-size $size -command "setValve $name" -fg lightblue
	label $frame.label -text $name -relief sunken -bd 1 -width 9 \
		-bg $color
	pack $frame.value -side top -pady 5
	pack $frame.pot -side top
	pack $frame.label -side top -expand y -fill y
}

proc addAlgos { frame } {
	global color

	frame $frame -relief sunken -bd 2 -bg $color
	frame $frame.bdy -relief flat -bd 0 -bg $color
	label $frame.lab -bg blue -relief sunken -bg black -fg green -textvariable \
		algo -width 12

	frame $frame.bdy.q
	potmeter $frame.bdy.q.pot -relief flat -bd 0 -bg $color -size 40 \
		-command "setValve Q" -style bevel -variable Q -fg lightblue
	label $frame.bdy.q.lab -bg blue -relief flat -bg $color -width 3 -text Q
	pack $frame.bdy.q.pot -side top
	pack $frame.bdy.q.lab -side top -expand y -fill both

	frame $frame.bdy.but -relief flat -bd 0 -bg $color
	radiobutton $frame.bdy.but.blue -indicator off -text Bv \
		-highlightthickness 0 -bg $color -variable algo -value BlueValve \
		-command "setValve algo 0"
	radiobutton $frame.bdy.but.vanzegg -indicator off -text Vz \
		-highlightthickness 0 -bg $color -variable algo \
		-value VanZeggelaars -command "setValve algo 1"
	radiobutton $frame.bdy.but.green -indicator off -text Gv \
		-highlightthickness 0 -bg $color -variable algo -value greenValve \
		-command "setValve algo 2"
#	radiobutton $frame.bdy.but.yellow -indicator off -text Yv 
#		-highlightthickness 0 -bg $color -variable algo -value yellowValve 
#		-command "setValve algo 3"
	checkbutton $frame.bdy.but.yellow -indicator off -text Inv \
		-highlightthickness 0 -bg $color -variable invert -onvalue invert \
		-command "setValve invert" -offvalue normal
	pack $frame.bdy.but.vanzegg -side top -expand y -fill x
	pack $frame.bdy.but.yellow -side bottom -expand y -fill x
	pack $frame.bdy.but.green -side bottom -expand y -fill x
	pack $frame.bdy.but.blue -anchor w -expand y -fill x

	pack $frame.lab -side top -pady 10 -padx 5
	pack $frame.bdy.but -side left -expand y -fill y
	pack $frame.bdy.q -side right -expand y -fill y
	pack $frame.bdy -side top -expand y -fill x
	pack $frame -side left -expand y -fill y
}

proc addMain { frame } {
	global color

	frame $frame -relief flat -bd 2 -bg $color
	frame $frame.gain -bg $color
	addPotMeter $frame.gain gain large
	pack $frame.gain -side left
	pack $frame -side left -expand y -fill y
}

proc addAmp { frame } {
	global color

	frame $frame -relief raised -bd 2 -bg $color
	frame $frame.algo -bg $color
	frame $frame.inputbias -bg $color
	frame $frame.outputbias -bg $color
#	frame $frame.q -bg $color

	addPotMeter $frame.inputbias inputbias medium
	addPotMeter $frame.outputbias outputbias medium
#	addPotMeter $frame.q Q medium

	pack $frame -side left
	pack $frame.algo -side left
	pack $frame.inputbias -side left
	pack $frame.outputbias -side left
#	pack $frame.q -side left
	pack $frame -side left -expand y -fill y
}

proc addLogo { frame } {
	global color

	set font "-adobe-symbol-*-*-*-*-*-250-*-*-*-*-*-*"

	frame $frame -relief raised -bd 2 -bg $color
	frame $frame.text -relief flat -bg $color
	label $frame.text.blue -relief flat -bg $color -text Blue -font $font \
		-fg red
	label $frame.text.valve -relief flat -bg $color -text valve -font $font \
		-fg red
	label $frame.text.it -relief flat -bg $color -text iT -fg red -font $font
	pack $frame.text.blue -side top -expand y -fill x
	pack $frame.text.valve -side top -expand y -fill x
	pack $frame.text.it -side top -expand y -fill x
	pack $frame.text -side left -anchor w -expand y -fill y
	pack $frame -side left -expand y -fill y
}

proc addEQ { frame } {
	global color

	frame $frame -relief raised -bd 2 -bg $color
	frame $frame.top
	label $frame.top.lab -relief flat -bg $color -text EQ
	frame $frame.top.sep -relief raised -bd 1
	frame $frame.bottom -relief flat -bg $color
	frame $frame.bottom.bass -bg $color
	frame $frame.bottom.middle -bg $color
	frame $frame.bottom.treble -bg $color

	addPotMeter $frame.bottom.bass bass medium
	addPotMeter $frame.bottom.middle middle medium
	addPotMeter $frame.bottom.treble treble medium

	pack $frame.bottom.bass -side left
	pack $frame.bottom.middle -side left
	pack $frame.bottom.treble -side left
	pack $frame.top.lab -side top -expand y -fill x
	pack $frame.top.sep -side top -expand y -fill x
	pack $frame.top -side top -expand y -fill x
	pack $frame.bottom -side bottom
	pack $frame -side left
}

proc setValvePreset { file } {
	global bvPre

	setValve preset $file $bvPre
}

proc getValveMem { file } {
	global bvMem

	setValve memget $file $bvMem
}

proc getValveMem2 { file } {
	global bvMem

	setValve memget $file $bvMem
}

proc setValveMem2 { file } {
	global bvMem

	setValve memset $file $bvMem
}

#
# Put presets into buttons:
# In the first case this will be to/from files, but eventually will compile them
# into the code.
#
# Notes: Memories and Preset are really the same, just that presets are loaded
# with parameter definitions. Both may be saved with new values specific to 
# a given mix.
#
proc buildPresets { frame } {
	global memoryFileName bvMem

	#
	# Want to get some values from a config file, if possible.
	#
	set memoryFileName [setValve getfilename].BlueValveIt

	frame $frame -bg blue

	radiobutton $frame.p0 -indicatoron false -text P0 -variable bvMem -value 4 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve presetget $memoryFileName 0"
	radiobutton $frame.p1 -indicatoron false -text P1 -variable bvMem -value 5 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve presetget $memoryFileName 1"
	radiobutton $frame.p2 -indicatoron false -text P2 -variable bvMem -value 6 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve presetget $memoryFileName 2"
	radiobutton $frame.p3 -indicatoron false -text P3 -variable bvMem -value 7 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve presetget $memoryFileName 3"
	button $frame.save -text Save -command "setValveMem2 $memoryFileName" \
		-highlightthickness 0 -bd 1 -bg blue -fg red -pady 0 -padx 0

	pack $frame.p0 $frame.p1 $frame.p2 $frame.p3 -side top \
		-expand y -fill both
	pack $frame.save -side bottom -padx 0 -pady 0 -expand n -fill both \
		-ipadx 0 -ipady 0

	pack $frame -side left -expand y -fill y -padx 3 -pady 3 -ipadx 3 -ipady 3

	#
	# Load the first file if possible
	#
	catch {setValve presetget $memoryFileName 0}
	set bvMem 0
}

#
# Put memories into buttons:
#
proc buildMemories { frame } {
	global memoryFileName bvMem

	#
	# Want to get some values from a config file, if possible.
	#
	set memoryFileName [setValve getfilename].BlueValveIt

	frame $frame -bg blue

	radiobutton $frame.m0 -indicatoron false -text M0 -variable bvMem -value 0 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve memget $memoryFileName 0"
	radiobutton $frame.m1 -indicatoron false -text M1 -variable bvMem -value 1 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve memget $memoryFileName 1"
	radiobutton $frame.m2 -indicatoron false -text M2 -variable bvMem -value 2 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve memget $memoryFileName 2"
	radiobutton $frame.m3 -indicatoron false -text M3 -variable bvMem -value 3 \
		-bg blue -highlightthickness 0 -selectcolor lightblue
#		-command "setValve memget $memoryFileName 3"
	button $frame.save -text Load -command "getValveMem2 $memoryFileName" \
		-highlightthickness 0 -bd 1 \
		-bg blue -fg red -pady 0 -padx 0

	pack $frame.m0 $frame.m1 $frame.m2 $frame.m3 -side top \
		-expand y -fill both
	pack $frame.save -side bottom -padx 0 -pady 0 -expand n -fill both \
		-ipadx 0 -ipady 0

	pack $frame -side left -expand y -fill y -padx 3 -pady 3 -ipadx 3 -ipady 3

	#
	# Load the first file if possible
	#
	catch {setValve memget $memoryFileName 0}
	set bvMem 0
}

proc buildMemoryMenu {} {
	global memoryFileName

	#
	# Want to get some values from a config file, if possible.
	#
	set memoryFileName [setValve getfilename].BlueValveIt

	menu .base.controls.mem
	.base.controls.mem add cascade -menu .base.controls.mem.load -label "Load"
	.base.controls.mem add cascade -menu .base.controls.mem.save -label "Save"
	menu .base.controls.mem.load
	.base.controls.mem.load add command -label "Memory 0" -command \
		"setValve memget $memoryFileName 0"
	.base.controls.mem.load add command -label "Memory 1" -command \
		"setValve memget $memoryFileName 1"
	.base.controls.mem.load add command -label "Memory 2" -command \
		"setValve memget $memoryFileName 2"
	.base.controls.mem.load add command -label "Memory 3" -command \
		"setValve memget $memoryFileName 3"
	menu .base.controls.mem.save
	.base.controls.mem.save add command -label "Memory 0" -command \
		"setValve memset $memoryFileName 0"
	.base.controls.mem.save add command -label "Memory 1" -command \
		"setValve memset $memoryFileName 1"
	.base.controls.mem.save add command -label "Memory 2" -command \
		"setValve memset $memoryFileName 2"
	.base.controls.mem.save add command -label "Memory 3" -command \
		"setValve memset $memoryFileName 3"

	bind SLabLogo <Button-3> {
		catch {tk_popup .base.controls.mem %X %Y}
	}

	#
	# Load the first file if possible
	#
	catch {setValve memget $memoryFileName 0}
}

proc buildPanel { frame } {
	addAmp $frame.amp
	addAlgos $frame.algos
	addEQ $frame.eq
	addMain $frame.gain
#	addLogo $frame.logo
	buildMemories $frame.mem
	buildPresets $frame.pre
}

wm colormap .

set color blue
set algo VanZeggelaars

# addLabels

image create photo BlueValveIt -file ../bitmaps/BlueValveIt.ppm

button .base -image BlueValveIt -width 750 -height 150
bindtags .base {SLabLogo 2 3 4}
frame .base.controls -relief raised -bd 1 -bg blue

buildPanel .base.controls
pack .base.controls -side left -anchor w -padx 40
pack .base -ipadx 55 -ipady 15 -expand y -fill both

wm title . "BlueValveIt"

