
#include "slabEffects.h"

/*
 * For historic reasons, the descriptor for bus attachments is int *.
 */
int *devPtr;

/*
 * Used by the user interface in the effects rack.
 */
#define EFFECT_NAME "chorus"
#define SHORT_NAME "Chorus"

/*
 * These are dependant on the effect - parameters etc. This chorus implements
 * 4 individual delay lines, with a MAX delay, at 44100 samples, of about
 * 400ms.
 */
#define HISTORY_SIZE 65536
#define CHORUS_SIZE 16384
#define INDEX1_START 0
#define INDEX2_START 16384
#define INDEX3_START 32768
#define INDEX4_START 49152

int history[HISTORY_SIZE];
int indexOut1 = 0;
int indexOut2 = 0;
int indexOut3 = 0;
int indexOut4 = 0;
int indexIn = 0;
int Speed1, Speed2, Speed3, Speed4;

int speed, breadth;
int Breadth = 128;

int currentBreadth1 = 0;
int currentBreadth2 = 0;
int currentBreadth3 = 0;
int currentBreadth4 = 0;
int countBreadth1 = 0;
int countBreadth2 = 0;
int countBreadth3 = 0;
int countBreadth4 = 0;
int changeBreadth1 = 1;
int changeBreadth2 = 1;
int changeBreadth3 = 1;
int changeBreadth4 = 1;

#define Min 1

int checkParams();

/*
 * When the user interface attaches an effect, it passes two parameters, no
 * switches. This is very rudementary, but will be extended:
 *
 * argv[1] is the controlbuffer segID, argv[2] should be the bus requested
 */
main(argc, argv)
int argc;
char *argv[];
{
	int *busData, count, i;

	if (argc < 3)
	{
		printf("%s: parameter problem\n", argv[0]);
		exit(-1);
	}

	/*
	 * seAttach() will register with the mixengine, and link up the shared
	 * memory control and data structures. It effectively reserves a bus.
	 * The SLAB_EFFECTS_MONO really refers to the first bus attachment. We
	 * can attach to a second one with the SLAB_EFFECTS_STEREO flag. It is
	 * only used as a switch to use a slightly different link up protocol.
	 */
	if ((devPtr = seAttach(atoi(argv[1]), atoi(argv[2]),
		SLAB_EFFECTS_MONO)) ==
		(int *) NULL)
	{
		printf("%s: could not attach to requested bus\n", argv[0]);
		exit(-2);
	}

	/*
	 * At this point we can use seAddCtrl() to request up to 8 parameters
	 * which will be shown in the SLab effects user interface. The chorus algo
	 * requires speed, breadth. The calls return an int which is
	 * used later when requesting paramter values.
	 *
	 * Interface only supports the SLab_Potmeter controller type now.
	 */
	speed = seAddCtrl(devPtr, "Speed", SLab_Potmeter);
	breadth = seAddCtrl(devPtr, "Breadth", SLab_Potmeter);

	/*
	 * seInit() returns our fixed buffer size. We always use a fixed buffer as
	 * an internal data representation. No exceptions. Data is manipulated in
	 * this fixed buffer, and then the buffer (pointer) is returned to the bus.
	 *
	 * The buffer size is configurable in the GUI, to allow some memory tuning.
	 * For this reason we need to be able to find out the current value.
	 */
	count = seInit(devPtr, EFFECT_NAME, SHORT_NAME, checkParams);

	/*
	 * This is a forever loop, sort of.....
	 */
	while ((busData = seRead(devPtr)) != (int *) -1) {
		chorus(busData, count);
		seWrite(devPtr, busData);
	}

	/*
	 * seDetach() will deregister with the mixengine, and remove links to the
	 * shared memory control and data structures.
	 */
	if (seDetach(devPtr) < 0)
	{
		printf("%s: could not detach from requested bus\n", argv[0]);
		exit(-3);
	}

	printf("Exiting\n");

	exit(0);
}

checkParams(devPtr, param)
int *devPtr, param;
{
	char text[32];
	int tmp;

	if ((tmp = (256 - seGetCtrlVal(devPtr, speed)) << 2) != Speed1)
	{
		Speed1 = (256 - seGetCtrlVal(devPtr, speed)) << 2;
		Speed2 = Speed1 + 10;
		Speed3 = Speed1 + 20;
		Speed4 = Speed1 + 30;
		sprintf(text, "Speed: %i", Speed1);
		seSetCtrlDisp(devPtr, speed, text);
	}
	Breadth = (seGetCtrlVal(devPtr, breadth) << 4);
	sprintf(text, "Breadth: %i", breadth);
	seSetCtrlDisp(devPtr, breadth, text);
}

chorus(busData, count)
register int *busData, count;
{
	for (; count > 0; count--)
	{
		if (++indexIn >= Breadth) indexIn = 0;

		history[indexIn] = 
		history[indexIn + INDEX2_START] =
		history[indexIn + INDEX3_START] =
		history[indexIn + INDEX4_START] = *busData;

		*(busData++) = history[indexOut1] +
			history[indexOut2 + INDEX2_START] +
			history[indexOut3 + INDEX3_START] +
			history[indexOut4 + INDEX4_START];

		if (++countBreadth1 >= Speed1)
		{
			countBreadth1 = 0;

			if ((currentBreadth1 += changeBreadth1) >= Breadth)
				changeBreadth1 = -1;
			if (currentBreadth1 <= Min)
				changeBreadth1 = 1;

			/*
			 * See if we are closing our window, skip a sample.
			 */
			if (changeBreadth1 < 0) {
				if ((indexOut1 += 1) >= Breadth) indexOut1 = 0;
				if ((indexOut1 += 1) >= Breadth) indexOut1 = 0;
			}
			/*
			 * We should add a check to make sure indexOut never overtakes
			 * indexIn? Preferable algorithm would slip indexOut backwards,
			 * and then pull it in until it was equal to indexIn, then start
			 * slipping again.
			 */
		} else {
			if (++indexOut1 >= Breadth) indexOut1 = 0;
		}

		if (++countBreadth2 >= Speed2)
		{
			countBreadth2 = 0;

			if ((currentBreadth2 += changeBreadth2) >= Breadth)
				changeBreadth2 = -1;
			if (currentBreadth2 <= Min)
				changeBreadth2 = 1;
			/*
			 * See if we are closing our window, skip a sample.
			 */
			if (changeBreadth2 < 0) {
				if ((indexOut2 += 1) >= Breadth) indexOut2 = 0;
				if ((indexOut2 += 1) >= Breadth) indexOut2 = 0;
			}
		} else {
			if (++indexOut2 >= Breadth) indexOut2 = 0;
		}

		if (++countBreadth3 >= Speed3)
		{
			countBreadth3 = 0;

			if ((currentBreadth3 += changeBreadth3) >= Breadth)
				changeBreadth3 = -1;
			if (currentBreadth3 <= Min)
				changeBreadth3 = 1;

			/*
			 * See if we are closing our window, skip a sample.
			 */
			if (changeBreadth3 < 0) {
				if ((indexOut3 += 1) >= Breadth) indexOut3 = 0;
				if ((indexOut3 += 1) >= Breadth) indexOut3 = 0;
			}
		} else {
			if (++indexOut3 >= Breadth) indexOut3 = 0;
		}

		if (++countBreadth4 >= Speed4)
		{
			countBreadth4 = 0;

			if ((currentBreadth4 += changeBreadth4) >= Breadth)
				changeBreadth4 = -1;
			if (currentBreadth4 <= Min)
				changeBreadth4 = 1;

			/*
			 * See if we are closing our window, skip a sample.
			 */
			if (changeBreadth4 < 0) {
				if ((indexOut4 += 1) >= Breadth) indexOut4 = 0;
				if ((indexOut4 += 1) >= Breadth) indexOut4 = 0;
			}
		} else {
			if (++indexOut4 >= Breadth) indexOut4 = 0;
		}
	} 
}

