
/*
 * file: interpreters.c
 * This should just contain code to encode and decode SLab message buffers.
 */

#include "slabmessages.h"

/*
 * Called by the signal driver buffer recv() manager.
 */
int
extractSLabMessage(buf, size)
slabmessage *buf;
{
#ifdef DEBUG
	printf("extractSLabMessage(%i)\n", size);
#endif

	if ((size < SLAB_MIN_MSG_SIZE) || (size > SLAB_MAX_MSG_SIZE))
		return(-1);
	/*
	 * Do what we can to ensure we recognise the frame
	 */
	if ((buf->S != 'S') ||
		(buf->L != 'L') ||
		(buf->a != 'a') ||
		(buf->b != 'b'))
		return(-2);

	switch (buf->version) {
		case 1:
			return(extractV1Message(buf, size));
			break;
		default:
			return(-3);
	}
	return(0);
}

extractV1Message(buf, size)
slabmessage *buf;
{
#ifdef DEBUG
	printf("extractV1Message(%i, comm: %i)\n", size, buf->command);
#endif
	/*
	 * Make sure we do not have a ridiculous message type.
	 */
	if ((buf->command < 0) || (buf->command >= SLAB_OP_MAX))
		return;

	return(interpreters[buf->command].decode(buf, size));
}

buildSLabMessage(msg, op, track, cont, operator, value)
slabmessage *msg;
{
	int result;

printf("buildSLabMessage(%i, %i, %i, %i, %i)\n",
op, track, cont, operator, value);

	if (op < 0 || op >= SLAB_OP_MAX)
		return(-1);

	msg->S = 'S';
	msg->L = 'L';
	msg->a = 'a';
	msg->b = 'b';

	msg->version = SLAB_CONTROL_VERSION;
	msg->command = op;

	interpreters[op].encode(msg, track, cont, operator, value);
}

buildTrackSetMessage(buf, track, controller, operator, value)
slabmessage *buf;
{
#ifdef DEBUG
printf("buildTrackSetMessage(%i, %i, %i, %i)\n",
track, controller, operator, value);
#endif

	/*
	 * Should really have parameter value checking, but not for now.....
	 */
	buf->message.trkMsg.track = track;
	buf->message.trkMsg.controller = controller;
	buf->message.trkMsg.operator = operator;
	buf->message.trkMsg.value = htons((unsigned short int) value);

printf("trackEvent: trk: %i, cont: %i, op %i, value %i\n",
buf->message.trkMsg.track,
buf->message.trkMsg.controller,
buf->message.trkMsg.operator,
ntohs((unsigned short int) buf->message.trkMsg.value));
}

/*
 * These routines are called by the server side, and use some routines in the
 * mixslab executable. They are not to be compiled into the client side.
 * Called with an initialised buffer with "SLab" and the command set up. We
 * need to put in any further information required.
 */
extractTrackSetMessage(msg, size)
slabmessage *msg;
{
#ifdef DEBUG
	printf("extractTrackSetMessage()\n");
#endif
	/*
	 * The track messages are of a fixed size.
	 */
	if (ntohs(msg->length) != sizeof(slabmessage))
		return;
	/*
	 * A couple of short parameters need to be converted.
	 */
	msg->length = ntohs(msg->length);
	msg->message.trkMsg.value = ntohs((unsigned short int) msg->message.trkMsg.value);

printf("trackEvent: trk: %i, cont: %i, op %i, value %i\n",
msg->message.trkMsg.track,
msg->message.trkMsg.controller,
msg->message.trkMsg.operator,
msg->message.trkMsg.value);
	return(0);
}

nullAPISubroutine()
{
#ifdef DEBUG
	printf("nullAPISubroutine()\n");
#endif
	return(-1);
}

buildMidiMessage(msg, channel, operation, time, param)
slabmessage *msg;
{
printf("buildMidiMessage(%i, %i, %i, %i)\n", channel, operation, time, param);

	if (operation != MIDI_CLOCK)
		return;
	/*
	 * Message already has the SLab header and version, we need to put the 
	 * midi specific parts in here, devending on operation code. This actually
	 * looks to require more parameters for a general operation?
	 */
	msg->message.midiMsg.channel = channel;
	msg->message.midiMsg.operation = operation;
	/*
	 * Time can be -1, in which case the server will put its own time in when
	 * the message arrives. Alternatively the client can put in the arrival 
	 * time of the MIDI message (just moves the responsibility
	 */
	msg->message.midiMsg.time = htonl(time);

printf("midiEvent: chan: %i, op: %i, time %i\n",
msg->message.midiMsg.channel,
msg->message.midiMsg.operation,
ntohl(msg->message.midiMsg.time));
}

extractMidiMessage(msg, channel, cont, operation, time)
slabmessage *msg;
{
printf("extractMidiMessage()\n");
	msg->message.midiMsg.time = ntohl(msg->message.midiMsg.time);
	return(0);
}

initLimits()
{
	int i;
	/*
	 * Zero the limits. If min and max are both zero, assume parameter is 0..255
	 */
	bzero(paramLimits, sizeof(struct limits) * LIST_END);
	for (i = 0; i < LIST_END; i++)
	{
		paramLimits[i].min = 0;
		paramLimits[i].max = 256;
	}
	/*
	 * For any unusual parameter ranges, put the min/max in here.
	 */
	paramLimits[GAIN].max = 100;
	paramLimits[PAN].min = -128;
	paramLimits[PAN].max = 128;
	paramLimits[MICROCD].max = 100;
	paramLimits[MICROPCM].max = 100;
	paramLimits[MICROMIC].max = 100;
	paramLimits[MICROSYNTH].max = 100;
	paramLimits[MICROLINE].max = 100;
	paramLimits[MICROLINE1].max = 100;
	paramLimits[MICROLINE2].max = 100;
	paramLimits[MICROLINE3].max = 100;

	paramLimits[BASSLIMIT].min = 1;
	paramLimits[BASSLIMIT].max = 64;
	paramLimits[BASSGAIN].max = 1024;
	paramLimits[BASSSWEEP].max = 32;
	paramLimits[BASSQ].max = 16;
	paramLimits[TREBLIMIT].min = 8;
	paramLimits[TREBGAIN].max = 1024;
	paramLimits[TREBSWEEP].max = 32;
	paramLimits[TREBQ].max = 16;
	paramLimits[PARALIM].min = 8;
	paramLimits[PARALIM].max = 164;
	paramLimits[PARAGAIN].max = 1024;
	paramLimits[PARAQ].max = 16;
}

/*
 * Put in any data required to manage the interpreter arrays.
 */
initInterpreters()
{
	int i;

#ifdef DEBUG
	printf("initInterpreters()\n");
#endif

	for (i = 0; i < SLAB_OP_MAX; i++)
	{
		interpreters[i].encode = nullAPISubroutine;
		interpreters[i].decode = nullAPISubroutine;
	}
	/*
	 * These routines are actually going to be common to a number of message
	 * types, since they converge onto exacly the same session event operations.
	 * Master, Track, Bus, StereoBus and FX operations should fit.
	 */
	interpreters[SLAB_T_SET].encode = buildTrackSetMessage;
	interpreters[SLAB_T_SET].decode = extractTrackSetMessage;

	interpreters[SLAB_M_SET].encode = buildTrackSetMessage;
	interpreters[SLAB_M_SET].decode = extractTrackSetMessage;

	interpreters[SLAB_B_SET].encode = buildTrackSetMessage;
	interpreters[SLAB_B_SET].decode = extractTrackSetMessage;

	interpreters[SLAB_SB_SET].encode = buildTrackSetMessage;
	interpreters[SLAB_SB_SET].decode = extractTrackSetMessage;

	interpreters[SLAB_MIDI_OP].encode = buildMidiMessage;
	interpreters[SLAB_MIDI_OP].decode = extractMidiMessage;
}

